Package: LearnPCA
Type: Package
Title: Functions, Data Sets and Vignettes to Aid in Learning Principal
        Components Analysis (PCA)
Version: 0.3.4
Date: 2024-04-25
Authors@R: c(
    person("Bryan A.", "Hanson",
      role = c("aut", "cre"),
      email = "hanson@depauw.edu",
      comment = c(ORCID = "0000-0003-3536-8246")),
    person("David T.", "Harvey",
      role = c("aut"),
      email = "harvey@depauw.edu"))
Description: Principal component analysis (PCA) is one of the most widely used data analysis techniques.  This package provides a series of vignettes explaining PCA starting from basic concepts. The primary purpose is to serve as a self-study resource for anyone wishing to understand PCA better. A few convenience functions are provided as well.
License: GPL-3
URL: https://bryanhanson.github.io/LearnPCA/
BugReports: https://github.com/bryanhanson/LearnPCA/issues
ByteCompile: TRUE
VignetteBuilder: knitr
Encoding: UTF-8
Imports: markdown, shiny, stats, graphics
Suggests: ChemoSpec, chemometrics, knitr, tinytest, roxut, rmarkdown,
        plot3D, ade4, plotrix, latex2exp, plotly, xtable, bookdown,
Depends: rpart, class, nnet
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2024-04-25 20:32:15 UTC; bryanh
Author: Bryan A. Hanson [aut, cre] (<https://orcid.org/0000-0003-3536-8246>),
  David T. Harvey [aut]
Maintainer: Bryan A. Hanson <hanson@depauw.edu>
Repository: CRAN
Date/Publication: 2024-04-26 09:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 13:46:25 UTC; unix
