# Generated by roxygen2: do not edit by hand

export(PCAtoXhat)
export(PCsearch)
export(XtoPCAtoXhat)
import(class)
import(markdown)
import(nnet)
import(rpart)
import(shiny)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,prcomp)
importFrom(stats,runif)
importFrom(stats,var)
