---
title:  "A Guide to the LearnPCA Package"
author:
  - David T. Harvey^[Professor of Chemistry & Biochemistry, DePauw University, Greencastle IN USA., harvey@depauw.edu]
  - Bryan A. Hanson^[Professor Emeritus of Chemistry & Biochemistry, DePauw University, Greencastle IN USA., hanson@depauw.edu]
date:  "`r Sys.Date()`"
output:
#    bookdown::html_document2: # use for pkgdown site
    bookdown::pdf_document2: # use for CRAN to keep size down; breaks GHA
      toc: no
      toc_depth: 2
      fig_caption: yes
      number_sections: false
      css: vignette.css
vignette: >
    %\VignetteIndexEntry{Vignette 01: A Guide to Learning PCA with LearnPCA (Start Here)}
    %\VignetteKeywords{PCA}
    %\VignettePackage{LearnPCA}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
#link-citations: yes
#bibliography: PCA.bib
#biblio-style: plain
pkgdown:
  as_is: true
---

<!-- This chunk inserts common info about all the vignettes -->

```{r, echo = FALSE, results = "asis"}
res <- knitr::knit_child("top_matter.md", quiet = TRUE)
cat(res, sep = '\n')
```



# Audience

Simply put, the audience for this package is people who want to learn about and more deeply understand Principal Components Analysis (PCA).  Let's acknowledge that we are not talking to mathematicians and statisticians.  We are talking to people in most any other discipline that want to improve their understanding of PCA.  PCA is used in many fields, so we have included examples from several different fields.  As PCA is the foundation of a number of related methods, people wanting to learn those variants will need a solid understanding of PCA before continuing.

# Why a Package?

In our careers as researchers, we have needed to use PCA, and hence, to understand PCA.  In our careers as professors of chemistry, we have taught PCA to students.  We can confirm that trying to teach PCA, and eventually getting better at it, is one of the best ways to learn PCA.

Even so, we do not pretend to be experts in PCA, though we are not novices either.  We have learned PCA like many of you, as a journey of gradually increasing understanding.  We feel like that gives us an advantage in explaining PCA to new learners, as we have been learners in the recent past.  In that sense PCA is like a lot of complex topics, it just cannot be fully understood in a single sitting.   So plan to read, practice, reflect and repeat.

So why put this material in package?  In the course of our own learning and teaching, we have tried to identify the best ways of teaching and illustrating this material.  `LearnPCA` is our attempt to gather those materials in an integrated, user-friendly way.  And we get a lot of questions from students.  We are glad to answer those questions, but we hope that this package will be useful to a wider range of PCA learners.

# About the Authors

Bryan Hanson is a freelance `R` consultant and software developer.  In a previous life he taught organic chemistry and biochemistry at DePauw University for 32 years.  He blames co-author David Harvey for dragging him down the `R` rabbit hole.  You can learn all about him at his [web site](http://ChemoSpec.org).

David Harvey is a faculty member in the Department of Chemistry & Biochemistry at DePauw University where he has worked since 1986. He claims no knowledge of rabbit holes and maintains his innocence in luring others into `R`. You can learn more about his interests at his [web site](https://sites.google.com/depauw.edu/dth/home).

# Acknowledgements

We have learned what we know from many articles, a few books, many blog posts and questions on StackExchange.  Where we rely heavily on particular sources, we will cite them in the appropriate place.  We will also point out additional useful resources for further study.

<!-- Insert works consulted document -->

```{r, echo = FALSE, results = "asis"}
res <- knitr::knit_child("works_consulted.md", quiet = TRUE)
cat(res, sep = '\n')
```
